package com.gic.cloud.data.hook.api.service;

import com.gic.cloud.data.hook.api.dto.*;

import java.util.List;

/** 自定义查询服务
 * @author Sam
 */
public interface IFreeQueryService {

	/** 获取所有自定义查询表
	 * @return
	 */
	public List<FreeQueryTable> getFreeQueryTableList();

	/** 创建新的自定义查询记录
	 * @param record
	 * @return
	 */
	public Integer createFreeQueryRecord(FreeQueryRecord record);

	/** 更新原有的自定义查询记录
	 * @param record
	 * @return
	 */
	public Integer updateFreeQueryRecord(FreeQueryRecord record);

	/** 删除一个自定义查询记录
	 * @param recordId
	 * @return
	 */
	public Integer deleteFreeQueryRecord(String recordId);

	/** 获取指定用户和编号的自定义查询记录
	 * @param userId
	 * @param recordId
	 * @return
	 */
	public FreeQueryRecord getFreeQueryRecordByUserIdAndRecordId(String userId, String recordId);

	/** 获取指定用户的所有自定义查询
	 * @param userId
	 * @param fuzzy
	 * @return
	 */
	public List<FreeQueryRecord> getFreeQueryRecordListByUserId(String userId, String fuzzy);

	/** 获取自定义查询总数
	 * @param sql
	 * @return
	 */
	public Integer getFreeQueryCount(String sql, String enterpriseId);

	/** 获取自定义查询结果
	 * @param sql
	 * @return
	 */
	public FreeQueryResult getFreeQueryResult(String sql, String enterpriseId, Integer queryDataType, Integer desensiType);

	/** 生成下载任务
	 * @param userId
	 * @param name
	 * @param amount
	 * @param format
	 * @param sql
	 * @return
	 */
	public DownloadTask buildFreeQueryDownloadTask(String userId, String name, Integer amount, String format, String sql, String enterpriseId, Boolean useCompress, Integer queryDataType, String applyId, Integer desensiType);


	/** 获取指定表的所有字段
	 * @param tableName
	 * @return
	 */
	public List<String> getColumnsByTable(String tableName);


	FreeQuerySource getFreeQuerySource(String enterpriseId);
}
