package com.gic.cloud.data.hook.service.impl;

import cn.medubi.client.utils.LogPak;
import com.alibaba.fastjson.JSON;
import com.gic.cloud.common.api.base.Page;
import com.gic.cloud.data.hook.api.dto.*;
import com.gic.cloud.data.hook.api.entity.*;
import com.gic.cloud.data.hook.api.service.IFlatQueryResultService;
import com.gic.cloud.data.hook.service.DecryptUtils;
import com.gic.cloud.data.hook.service.HiveHelper;
import com.gic.cloud.data.hook.service.HttpUtils;
import com.gic.cloud.data.hook.service.ServiceUtils;
import com.gic.cloud.data.hook.service.dao.FlatQueryTableDao;
import com.gic.cloud.data.hook.service.entity.CsvDataFilterMode;
import com.gic.cloud.data.hook.service.entity.CsvResultSetHelper;

import com.google.common.collect.Lists;
import com.opencsv.CSVWriter;
import com.opencsv.ResultSetHelper;
import io.netty.handler.codec.http.HttpUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.sql.DataSource;
import java.io.*;
import java.math.BigDecimal;
import java.nio.charset.Charset;
import java.sql.*;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.Date;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/** 自助指标查询服务实现
 * @author Sam
 */
public class FlatQueryResultServiceImpl implements IFlatQueryResultService {

	/** 日志类 */
	private static LogPak log = new LogPak(FlatQueryResultServiceImpl.class);

	/** csv / xls 下载目录 */
	public static final String SAVE_FOLDER = "/usr/local/data-hook-file";

	public static final List<String> PHONE = Arrays.asList("mobile", "phone", "phone_number", "receive_phone_number");

	public static final List<String> CARD = Arrays.asList("card_num", "receive_card_num");

	@Autowired
	FlatQueryTableDao flatQueryTableDao;

	/** 自助指标查询关联的下载条件列表 */
	protected List<FlatQueryTaskCondition> taskConditions = Lists.newArrayList();

	private FlatQueryResultServiceImpl() {
		log.debug("construct", "准备初始化 FlatQuery 查询服务");
		runDownloadTask(3);
		runApplyTask(5); // 每5秒钟进行任务状态检测
	}

	/** 生成自助指标查询最终 SQL
	 * @param queryOrDownload 查询或下载用途
	 * @param tableId
	 * @param enterpriseIds
	 * @param queryConditions
	 * @param orderField
	 * @param orderDir
	 * @param execDistinct
	 * @param limitRange
	 * @return
	 */
	public String buildFlatQuerySQL(Boolean queryOrDownload, String tableId, List<String> enterpriseIds, List<FlatQueryCondition> queryConditions, String orderField, String orderDir,
	                                Boolean execDistinct, Integer limitRange, List<String> authStoreIdList) {
		log.debug("buildFlatQuerySQL", "自助指标查询来自 " + enterpriseIds.get(0));
		// 查询处理开始
		String queryDistinct = execDistinct ? " DISTINCT " : "";
		//String queryOrder = StringUtils.isNotBlank(orderField) ? " ORDER BY \"" + orderField + "\" " : "";
		String queryOrder = StringUtils.isNotBlank(orderField) ? " ORDER BY " + orderField + " " : "";
		String queryDir = StringUtils.isNotBlank(orderDir) ? " " + orderDir + " " : "";
		String queryColumns = "";
		String queryFilters = "";
		if (enterpriseIds.size() == 1) { // 如果是单个企业
			queryFilters = " WHERE enterprise_id = '" + enterpriseIds.get(0) + "' ";
		} else { // 如果是多个企业
			for (int i=0; i<enterpriseIds.size(); i++) {
				queryFilters += queryFilters.equals("") ? " '" + enterpriseIds.get(i) + "' " : ", '" + enterpriseIds.get(i) + "' ";
			} // FOR OVER
			queryFilters = " WHERE enterprise_id IN ( " + queryFilters + " )  ";
		} // IF ELSE OVER
		for (FlatQueryCondition queryCondition : queryConditions) {
			//queryColumns += queryColumns.equals("") ? " \"" + queryCondition.getFieldName() + "\" " : ", \"" + queryCondition.getFieldName() + "\" ";
			if (queryOrDownload) { // 如果是查询
				queryColumns += queryColumns.equals("") ? " " + queryCondition.getFieldName() + " " : ", " + queryCondition.getFieldName() + " ";
			} else { // 如果是下载
				queryColumns += queryColumns.equals("")
						? " " + queryCondition.getFieldName() + " AS `" + queryCondition.getFieldMark().trim() + "` "
						: ", " + queryCondition.getFieldName() + " AS `" + queryCondition.getFieldMark().trim() + "` ";
			} // IF ELSE OVER
			// 如果存在主过滤器的值，视为过滤器生效
			if (StringUtils.isNotBlank(queryCondition.getMainFilter())) {
				String curFilterText = "";

				// 文本型字段处理
				// 文本型字段暂时只使用 mainFilter，以 | 分割关键字绑定
				if (queryCondition.getFieldType().equals(FlatQueryFieldType.TEXT)) {
					log.debug("buildFlatQuerySQL", "文本型字段：" + queryCondition.getFieldName());
					String[] equalValues = queryCondition.getMainFilter().split("\\|");
					if (equalValues.length > 0) { // 如果有过滤值
						//curFilterText = "\"" + queryCondition.getFieldName() + "\" in (";
						curFilterText = " " + queryCondition.getFieldName() + " in (";
						for (int i = 0; i < equalValues.length; i++) {
							curFilterText += i == 0 ? "'" + equalValues[i] + "'" : ",'" + equalValues[i] + "'";
						}
						curFilterText += ") ";
					} // IF OVER
				}

				// 数值型字段处理
				else if (queryCondition.getFieldType().equals(FlatQueryFieldType.NUMBER)) {
					log.debug("buildFlatQuerySQL", "数值型字段：" + queryCondition.getFieldName());
					curFilterText += " (";
					//curFilterText += "\"" + queryCondition.getFieldName() + "\"" + queryCondition.getMainFilter() + " ";
					curFilterText += " " + queryCondition.getFieldName() + " " + queryCondition.getMainFilter() + " ";
					if (!queryCondition.getFilterTunnel().equals(FilterTunnelType.NONE)) {
						//curFilterText += " " + queryCondition.getFilterTunnel() + " \"" + queryCondition.getFieldName() + "\"" + queryCondition.getMainFilter() + " ";
						curFilterText += " " + queryCondition.getFilterTunnel() + " " + queryCondition.getFieldName() + " " + queryCondition.getExtendFilter() + " ";
					} // IF OVER
					curFilterText += ") ";
				}

				// 时间型字段处理
				// 时间型字段无需处理前后日期差别
				else if (queryCondition.getFieldType().equals(FlatQueryFieldType.DATE)) {
					log.debug("buildFlatQuerySQL", "日期型字段：" + queryCondition.getFieldName());
					//String startDate = queryCondition.getMainFilter() + " 00:00:00";
					//String overDate = queryCondition.getExtendFilter().equals("") ? queryCondition.getMainFilter() + " 59:59:59" : queryCondition.getExtendFilter() + " 59:59:59";
					String startDate = queryCondition.getMainFilter() + "";
					String overDate = queryCondition.getExtendFilter() + "";
					//curFilterText = "\"" + queryCondition.getFieldName() + "\" >= '" + startDate + "' AND \"" + queryCondition.getFieldName() + "\" <= '" + overDate + "'";
					curFilterText = " " + queryCondition.getFieldName() + " >= '" + startDate + "' AND " + queryCondition.getFieldName() + " <= '" + overDate + "'";
				}

				// 合并到主查询过滤中
				queryFilters += queryFilters.equals("") ? " " + curFilterText + " " : " AND " + curFilterText + " ";

			} // IF OVER
		} // FOR OVER

		//管辖门店权限控制
		log.debug("buildAuthQuery", JSON.toJSONString(authStoreIdList));
		if(CollectionUtils.isNotEmpty(authStoreIdList)){
			FlatQueryTable table = this.flatQueryTableDao.getFlatQueryTableById(null, tableId);
			if(table != null){
				queryFilters += " AND " + table.getAuthStoreId() + " in(";
				for(int i=0 ; i< authStoreIdList.size(); i++){
					queryFilters += i == 0 ? "'" + authStoreIdList.get(i) + "'" : ",'" + authStoreIdList.get(i) + "'";
				}
				queryFilters += ")";
			}
		}
		log.debug("buildAuthQueryResult", queryFilters);
		// 最终处理 queryFilters、LIMIT、DISTINCT
		if (!StringUtils.isNotBlank(queryFilters)) queryFilters = " AND " + queryFilters; // if (!queryFilters.equals("")) queryFilters = " WHERE " + queryFilters;
		String queryLimit = limitRange > 0 ? " limit " + limitRange : "";
		// 最终处理 SQL
		String queryFull = "SELECT " + queryDistinct + queryColumns + " FROM " + tableId + " " + queryFilters + " " + queryOrder + " " + queryDir + queryLimit;
		log.debug("buildFlatQuerySQL","生成自助指标查询SQL：" + queryFull);
		return queryFull;
	}

	@Override
	public FlatQueryResult getFlatQueryResult(String tableId, List<String> enterpriseIds, List<FlatQueryCondition> queryConditions, String orderField, String orderDir,
	                                          Boolean execDistinct, Integer queryDataType, Integer limitRange, int dataPermission, List<String> authStoreIdList) {
		log.debug("getFlatQueryResult", "自助指标查询来自 " + enterpriseIds.get(0));
		// 定义返回值
		FlatQueryResult result = new FlatQueryResult();
		// 定义查询语句
		String queryFull = this.buildFlatQuerySQL(
				true, // 查询用途
				tableId,
				enterpriseIds,
				queryConditions,
				orderField,
				orderDir,
				execDistinct,
				limitRange,
				authStoreIdList);
		// 进行查询
		Connection conn = HiveHelper.getHiveConnection();
		if (conn != null) {
			Date startTime = new Date();
			try {
				Statement stat = conn.createStatement();
				stat.execute("REFRESH TABLE " + tableId); // 强制刷新表信息
				ResultSet rs = stat.executeQuery(queryFull);
				ResultSetMetaData metaData = rs.getMetaData();
				List<String> cFilters = Lists.newArrayList();
				for (FlatQueryCondition cond : queryConditions) {
					if (cond.getEnableEncrypt()) {
						cFilters.add(cond.getFieldMark());
						log.debug("getFlatQueryResult", "新增查询加密字段条件：" + cond.getFieldMark());
					} // IF OVER
				} // FOR OVER
				while (rs.next()) {
					for (int i = 0; i < queryConditions.size(); i++) {
						FlatQueryCondition condition = queryConditions.get(i);
						String fieldName = condition.getFieldName();
//						if (cFilters.contains(fieldName)) {
//							if (queryDataType.equals(QueryDataType.FULL)) {
//								String preResult = rs.getString(fieldName);
//								String expResult = preResult != null ? DecryptUtils.getInstance().decrypt(preResult) : "";
//								result.add(fieldName, expResult);
//							} else result.add(fieldName, "******");
//						} else {
							int fieldColumnIndex = rs.findColumn(fieldName);
							int fieldType = rs.getMetaData().getColumnType(fieldColumnIndex);
							Object fieldResult = null;
							switch (fieldType) {
								case Types.DECIMAL:
									Object tmpDecimal = rs.getObject(fieldColumnIndex);
									fieldResult = tmpDecimal != null ? String.valueOf(rs.getBigDecimal(fieldColumnIndex)) : null;
									break;
								case Types.FLOAT:
								case Types.DOUBLE:
									Object tmpDouble = rs.getObject(fieldColumnIndex);
									fieldResult = tmpDouble != null ? String.valueOf(rs.getDouble(fieldColumnIndex)) : null;
									break;
								case Types.INTEGER:
									Object tmpInt = rs.getObject(fieldColumnIndex);
									fieldResult = tmpInt != null ? String.valueOf(rs.getInt(fieldColumnIndex)) : null;
									break;
								case Types.DATE:
									Date tmpDate = rs.getDate(fieldColumnIndex);
									fieldResult = tmpDate != null ? tmpDate.getTime() : null;
									break;
								default:
									// 文本型字符串判断是否需要解密
//									if (queryDataType == QueryDataType.FULL && condition.getEnableEncrypt()) { // 如果需要进行解密
//										String preResult = rs.getString(fieldColumnIndex);
//										//fieldResult = DecryptUtils.getInstance().decrypt(preResult);
//										if (StringUtils.isNotBlank(preResult)) {
//											fieldResult = DecryptUtils.decrypt(preResult);
//										} else fieldResult = "";
//									} else fieldResult = rs.getObject(fieldColumnIndex);
									//fieldResult = rs.getObject(fieldColumnIndex);
									/**if (condition.getEnableEncrypt()) { // 如果需要进行解密
										String preResult = rs.getString(fieldColumnIndex);
										//fieldResult = DecryptUtils.getInstance().decrypt(preResult);
										if (queryDataType == QueryDataType.FULL) {
											if (StringUtils.isNotBlank(preResult)) {
												fieldResult = DecryptUtils.decrypt(preResult);
											} else fieldResult = "";
										} else fieldResult = "******";
									} else fieldResult = rs.getObject(fieldColumnIndex);**/
									if (FreeQueryServiceImpl.FILTERS_PHONE_ONLY.contains(fieldName) || FreeQueryServiceImpl.FILTERS_PHONE_AND_CARD.contains(fieldName)) {
										if(queryDataType == QueryDataType.FULL){
											String preResult = rs.getString(fieldColumnIndex);
											if(dataPermission == 1 && FreeQueryServiceImpl.FILTERS_PHONE_ONLY.contains(fieldName)){
												fieldResult = DecryptUtils.decrypt(preResult);
											}else if(dataPermission == 2 && FreeQueryServiceImpl.FILTERS_PHONE_AND_CARD.contains(fieldName)){
												fieldResult = DecryptUtils.decrypt(preResult);
											}else fieldResult = rs.getObject(fieldColumnIndex);
										} else {
											if(dataPermission == 1 && FreeQueryServiceImpl.FILTERS_PHONE_ONLY.contains(fieldName)){
												fieldResult = "******";
											}else if(dataPermission == 2 && FreeQueryServiceImpl.FILTERS_PHONE_AND_CARD.contains(fieldName)){
												fieldResult = "******";
											} else fieldResult = rs.getObject(fieldColumnIndex);
										}
									} else fieldResult = rs.getObject(fieldColumnIndex);
							} // SWITCH OVER
							result.add(fieldName, fieldResult);
//						} // IF ELSE OVER
					} // FOR OVER
				} // WHILE OVER
			} catch (Exception ex) {
				ex.printStackTrace();
			} finally {
				Date endTime = new Date();
				log.debug("getFlatQueryResult", "自助指标执行时间：" + ((endTime.getTime() - startTime.getTime()) + " 毫秒"));
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		} // IF OVER
		// 返回结果
		return result;
	}

	/** 获取自助指标字段过滤条件数据
	 * @param tableId
	 * @param fieldName
	 * @return
	 */
	public List<Object> getFlatQueryFilterData(String tableId, List<String> enterpriseIds, String fieldName) {
		List<Object> result = Lists.newArrayList();
		Connection conn = HiveHelper.getHiveConnection();
		if (conn != null) {
			try {
				Statement stat = conn.createStatement();
				stat.execute("REFRESH TABLE " + tableId); // 强制刷新表结构
				// ff8080816133365501614f100ddf6ed7 江南布衣
				//String sql = "SELECT DISTINCT " + fieldName + " FROM (SELECT " + fieldName + " FROM " + tableId + " WHERE enterprise_id = 'ff8080816133365501614f100ddf6ed7') ";
				String sql = "SELECT DISTINCT " + fieldName + " FROM (SELECT " + fieldName + " FROM " + tableId + " WHERE enterprise_id = '" + enterpriseIds.get(0) + "') ";
				//String sql = "SELECT DISTINCT " + fieldName + " FROM " + tableId + " LIMIT 1000 ";
//				String sql = "with my_table as (" +
//						"select " + fieldName + " from " + tableId + " limit 1000" +
//						")" +
//						" select distinct " + fieldName + " from my_table";
				log.debug("自助指标字段过滤请求：", sql);
				ResultSet rs = stat.executeQuery(sql);
				while (rs.next()) {
					result.add(rs.getObject(fieldName));
				} // WHILE OVER
			} catch (Exception ex) {
				ex.printStackTrace();
			} finally {
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		} // IF OVER
		return result;
	}

	/** 评估自助指标查询下载任务数据量
	 * @param tableId
	 * @param queryConditions
	 * @param orderField
	 * @param orderDir
	 * @param execDistinct
	 * @return
	 */
	public Integer estimateFlatQueryDownloadTask(String tableId, List<String> enterpriseIds,
												 List<FlatQueryCondition> queryConditions, String orderField,
												 String orderDir, Boolean execDistinct, List<String> authStoreIdList) {
		Integer result = -1;
		// 生成正式查询
		String preQuery = this.buildFlatQuerySQL(
				true, // 查询用途
				tableId,
				enterpriseIds,
				queryConditions,
				orderField,
				orderDir,
				execDistinct,
				0,
				authStoreIdList);
		String finQuery = "SELECT COUNT(1) FROM (" + preQuery + ") AS tmpTable";
		Connection conn = HiveHelper.getHiveConnection();
		if (conn != null) {
			try {
				Statement stat = conn.createStatement();
				stat.execute("REFRESH TABLE " + tableId); // 强制刷新表结构
				ResultSet rs = stat.executeQuery(finQuery);
				if (rs.next()) {
					result = rs.getInt(1);
				}
			} catch (Exception ex) {
				ex.printStackTrace();
//				String exWord = ex.getMessage();
//				// Invalid SessionHandle
//				if (exWord.contains("HiveSQLException: Invalid SessionHandle")) {
//					DataSource source = HiveHelper.getDataSource();
//					source = new
//				}
			} finally {
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		} // IF OVER
		return result;
	}

	/** 创建自助指标查询的下载任务
	 * @param userId
	 * @param name
	 * @param amount
	 * @param format
	 * @param tableId
	 * @param enterpriseIds
	 * @param queryConditions
	 * @param orderField
	 * @param orderDir
	 * @param execDistinct
	 * @return
	 */
	public DownloadTask buildFlatQueryDownloadTask(String userId, String name, String moduleName, Integer amount, String format, String tableId, List<String> enterpriseIds, List<FlatQueryCondition> queryConditions, String orderField, String orderDir, Boolean execDistinct,
			String useCompress, Integer queryDataType, String applyId, List<String> authStoreIdList) {
		log.debug("buildFlatQueryDownloadTask", "自助指标下载请求来自 " + enterpriseIds.get(0));
		// 增加下载量
		FlatQueryTable table = this.flatQueryTableDao.getFlatQueryTableById(userId, tableId);
		if (table != null) {
			table.setDownloadCount(table.getDownloadCount() + 1);
			this.flatQueryTableDao.updateFlatQueryTable(table);
		}
		// 注册下载任务
		DownloadTask curTask = new DownloadTask();
		curTask.setId(String.valueOf((new Date()).getTime()));
		curTask.setQueryDataSource(QueryDataSource.FLAT_QUERY); // 设置为平面查询
		curTask.setUserId(userId);
		curTask.setName(name);
		curTask.setModuleName(moduleName);
		curTask.setAmount(amount);
		curTask.setFormat(format);
		curTask.setStartTime(new Date());
		curTask.setStatus(DownloadTaskStatus.WAITING);
		// 新增下载任务属性
		curTask.setUseCompress(useCompress);
		curTask.setApplyId(applyId); // 设置申请编号
		curTask.setQueryDataType(queryDataType);
		if (queryDataType == QueryDataType.SAFE) { // 如果是安全模式
			curTask.setApplyPermitted(Global.YES); // 安全模式直接允许下载
			curTask.setApplyStatus(DownloadApplyStatus.PERMITTED_BY_SELF);
		} else { // 如果是完整模式
			curTask.setApplyPermitted(Global.NO); // 完整模式默认不允许
			curTask.setApplyStatus(DownloadApplyStatus.WAITING);
		} // IF ELSE OVER
		curTask.setRealSql(""); // 设置查询为空
		curTask.setEnterpriseId(enterpriseIds.get(0));
		Integer preReportCount = DownloadTaskServiceImpl.getInstance().getDownloadTaskCountByEnterpriseId(curTask.getEnterpriseId());
		curTask.setReportId(ServiceUtils.buildReportId(curTask.getQueryDataType(), preReportCount));

		// 保存下载任务
		DownloadTaskServiceImpl.getInstance().createDownloadTask(curTask);

		// 生成条件
		FlatQueryTaskCondition condition = new FlatQueryTaskCondition();
		condition.setTaskId(curTask.getId());
		condition.setTableId(tableId);
		condition.setQueryDataType(queryDataType); // 新增申请的数据类型
		condition.setEnterpriseIds(enterpriseIds);
		condition.setConditions(queryConditions);
		condition.setOrderField(orderField);
		condition.setOrderDir(orderDir);
		condition.setExecDistinct(execDistinct);
		// 设置条件可否执行
		if (curTask.getApplyPermitted().equals(Global.YES)) {
			condition.setBuildPermitted(Global.YES);
		} else condition.setBuildPermitted(Global.NO);
		//设置管辖门店权限
		condition.setAuthStoreIdList(authStoreIdList);
		// 保存至列表
		this.taskConditions.add(condition);
		return curTask;
	}



	private static SimpleDateFormat datetimeFormatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	private static SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");

	private static SimpleDateFormat timeFormatter = new SimpleDateFormat("HH:mm:ss");


	/** 下载任务执行计时器 */
	private Timer downloadTaskTimer = new Timer();

	/** 启动自助指标查询计划任务 */
	private void runDownloadTask(Integer interval) {
		this.downloadTaskTimer.schedule(new TimerTask() {
			@Override
			public void run() {
				if (taskConditions != null && taskConditions.size() > 0) {
					FlatQueryTaskCondition condition = null;
					//FlatQueryTaskCondition condition = taskConditions.remove(0); // 移除并获取第一个任务条件
					for (int i=0; i<taskConditions.size(); i++ ) {
						if (taskConditions.get(i).getBuildPermitted().equals(Global.YES)) {
							condition = taskConditions.remove(i); // 移除并获取第一个任务条件
							break;
						} // IF OVER
					} // FOR OVER
					if (condition != null) {
						// 更新任务状态
						DownloadTask task = DownloadTaskServiceImpl.getInstance().getDownloadTaskById(condition.getTaskId());
						task.setStatus(DownloadTaskStatus.BUILDING);
						DownloadTaskServiceImpl.getInstance().updateDownloadTask(task);
						log.debug("runDownloadTask.run", "自助指标下载任务执行：" + task.getId());
						String fullQuery = buildFlatQuerySQL(
								false, // 下载用途
								condition.getTableId(),
								condition.getEnterpriseIds(),
								condition.getConditions(),
								condition.getOrderField(),
								condition.getOrderDir(),
								condition.getExecDistinct(),
								0,
								condition.getAuthStoreIdList());

						Connection conn = HiveHelper.getDownloadHiveConnection();
						if (conn != null) {
							try {
								Statement stat = conn.createStatement();

//								stat.setQueryTimeout(60 * 1000);
								stat.execute("REFRESH TABLE " + condition.getTableId()); // 强制刷新表结构
								ResultSet rs = stat.executeQuery(fullQuery);
								// 生成指定格式下载元文件
								String originalFilePath = "";
								if (task.getFormat().equals(DownloadFileFormat.CSV)) { // 如果指定为 CSV 格式
									log.debug("runDownloadTask.run", "准备生成自助指标下载文件 " + condition.getTaskId() + ".csv");
									originalFilePath = SAVE_FOLDER + "/" + condition.getTaskId() + ".csv";
									File tmp = new File(originalFilePath);
									if (tmp.exists()) { // 删除可能存在的文件
										tmp.delete();
									}
									//CSVWriter csvWriter = new CSVWriter(new FileWriter(csvPath), '\t');
									OutputStreamWriter out = new OutputStreamWriter(new FileOutputStream(originalFilePath), Charset.forName("GBK"));
									ResultSetHelper helper = new CsvResultSetHelper(task.getQueryDataType() == QueryDataType.FULL ? CsvDataFilterMode.DECRYPT : CsvDataFilterMode.DESENSI, condition.getDecryptFilters());
									CSVWriter writer = new CSVWriter(out, ',');
									writer.setResultService(helper);
									writer.writeAll(rs, true);
									writer.close();
									out.close();//记得关闭资源
									log.debug("runDownloadTask.run", "已生成自助指标下载文件 " + condition.getTaskId() + ".csv");
								} else { // 如果指定为 XLS 格式
									log.debug("runDownloadTask.run", "准备生成自助指标下载文件 " + condition.getTaskId() + ".xlsx");
									originalFilePath = SAVE_FOLDER + "/" + condition.getTaskId() + ".xlsx";
									SXSSFWorkbook wb = new SXSSFWorkbook(100); // 内存中保留 100 行
									Sheet sheet = wb.createSheet();
									Row row = sheet.createRow(0);
									Cell cell;
									for (int j = 0; j < rs.getMetaData().getColumnCount(); ++j) { // 遍历创建表头
										String colName = rs.getMetaData().getColumnLabel(j + 1);
										cell = row.createCell(j);
										cell.setCellValue(colName);
									}
									// 遍历输出行
									int rowCount = 0;
									while (rs.next()) {
										rowCount++;
										row = sheet.createRow(rowCount);
										for (int j = 0; j < rs.getMetaData().getColumnCount(); ++j) {
											//String c = rs.getString(j + 1);
											//row.createCell(j).setCellValue(c);
											String cName = rs.getMetaData().getColumnName(j+1);
											List<String> cFilters = condition.getDecryptFilters();
											if (task.getQueryDataType() == QueryDataType.FULL && cFilters.contains(cName)) {
												String tmpResult = rs.getString(j + 1);
												if (StringUtils.isNotBlank(tmpResult)) tmpResult = DecryptUtils.getInstance().decrypt(tmpResult);
												row.createCell(j).setCellValue(tmpResult);
											} else {
												int cType = rs.getMetaData().getColumnType(j + 1);
												switch (cType) {
													case Types.TIMESTAMP:
														row.createCell(j).setCellValue(rs.getTimestamp(j + 1) != null ? datetimeFormatter.format(rs.getTimestamp(j + 1)) : "");
														break;
													case Types.DATE:
														row.createCell(j).setCellValue(rs.getDate(j + 1) != null ? dateFormatter.format(rs.getDate(j + 1)) : "");
														break;
													case Types.TIME:
														row.createCell(j).setCellValue(rs.getTimestamp(j + 1) != null ? timeFormatter.format(rs.getTimestamp(j + 1)) : "");
														break;
													default:
														if(cFilters.contains(cName)){
															row.createCell(j).setCellValue("******");
														}else {
															row.createCell(j).setCellValue(rs.getString(j + 1));
														}
														break;
												}
											} // IF ELSE OVER
										} // FOR OVER
									} // WHILE OVER
									FileOutputStream fileOut = new FileOutputStream(originalFilePath);
									wb.write(fileOut);
									//fileOut.flush(); // SXSSFWorkbook 使用 auto-flush 模式
									fileOut.close();
									//wb.close();
									wb.dispose(); // SXSSFWorkbook 没有 close 方法
									log.debug("runDownloadTask.run", "已生成自助指标下载文件 " + condition.getTaskId() + ".xlsx");
								} // IF ELSE OVER

								// 如果指定压缩，则使用之
								//if (task.getFormat().equals("zip")) {
								if (task.getUseCompress().equals(Global.YES)) {
									log.debug("runDownloadTask.run", "准备生成自助指标压缩文件 " + condition.getTaskId() + ".zip");
									String zipFilePath = SAVE_FOLDER + "/" + condition.getTaskId() + ".zip";
									File zipFile = new File(zipFilePath);
									ZipOutputStream zos = null;
									byte[] buf = new byte[1024];
									int length = 0;
									try {
										OutputStream os = new FileOutputStream(zipFilePath);
										BufferedOutputStream bos = new BufferedOutputStream(os);
										zos = new ZipOutputStream(bos);
										zos.setLevel(6); // 压缩率选择 0-9
										InputStream is = new FileInputStream(originalFilePath);
										BufferedInputStream bis = new BufferedInputStream(is);
										zos.putNextEntry(new ZipEntry(originalFilePath.substring(originalFilePath.lastIndexOf("/") + 1)));
										while ((length = bis.read(buf)) > 0) {
											zos.write(buf, 0, length);
										}
										bis.close();
										is.close();
										//bos.close();
										//os.close();
										log.debug("runDownloadTask.run", "已生成自助指标压缩文件 " + condition.getTaskId() + ".zip");
									} catch (Exception ex2) {
										throw ex2;
									} finally {
										zos.closeEntry();
										zos.close();
									}
								}


								task.setStatus(DownloadTaskStatus.COMPLISHED);
								task.setOverTime(new Date());
								String taskFileExt = task.getUseCompress().equals(Global.YES) ? ".zip" : task.getFormat().equals(DownloadFileFormat.CSV) ? ".csv" : ".xlsx";
								task.setFilePath(task.getId() + taskFileExt);
								DownloadTaskServiceImpl.getInstance().updateDownloadTask(task);

							} catch (Exception ex) {
								ex.printStackTrace();
								// 标记任务异常
								task.setStatus(DownloadTaskStatus.ERROR);
								task.setOverTime(new Date());
								DownloadTaskServiceImpl.getInstance().updateDownloadTask(task);
							} finally {
								try {
									conn.close();
								} catch (SQLException e) {
									e.printStackTrace();
								}
							}

						} // IF OVER
					} // IF OVER
				} // 没有任务则忽略
			} // run Define Over
		}, interval * 1000, interval * 1000);// 配置中的值为毫秒
	}


	/** 下载申请检查计时器 */
	private Timer applyTimer = new Timer();

	/** 运行下载申请任务
	 * @param interval
	 */
	private void runApplyTask(Integer interval) {
		this.applyTimer.schedule(new TimerTask() {
			@Override
			public void run() {
				List<DownloadTask> waitingTasks = DownloadTaskServiceImpl.getInstance().getDownloadTaskOfWaiting(QueryDataSource.FLAT_QUERY);
				for (DownloadTask task : waitingTasks) {
					if (!task.getApplyId().equals("")) {
//						String applyStatusText = HttpUtils.doGet("http://www.gicdev.com/api-admin/apply-info?type=2&applyId=" + task.getApplyId());
						String applyStatusText = HttpUtils.doGet("http://hope.demogic.com/api-admin/apply-info?type=2&applyId=" + task.getApplyId());
						log.debug("runApplyTask.run", "查询自助指标查询 " + task.getId() + " 审核状态码" + applyStatusText);
						boolean conditionRemove = false; // 从上传条件移除
						boolean conditionUpdate = false; // 更新条件
						if (applyStatusText.indexOf("\"checkStatus\":0") > 0) { // 如果自审核通过
							task.setApplyStatus(DownloadApplyStatus.PERMITTED_BY_SELF);
							task.setApplyPermitted(Global.YES);
							conditionUpdate = true;
						} else if (applyStatusText.indexOf("\"checkStatus\":2") > 0) { // 如果第三方审核通过
							task.setApplyStatus(DownloadApplyStatus.PERMITTED_BY_OTHER);
							task.setApplyPermitted(Global.YES);
							conditionUpdate = true;
						} else if (applyStatusText.indexOf("\"checkStatus\":1") > 0) { // 如果审核超时
							task.setApplyStatus(DownloadApplyStatus.TIMEOUT);
							task.setApplyPermitted(Global.NO);
							conditionRemove = true;
						}
						if (conditionRemove || conditionUpdate) {
							DownloadTaskServiceImpl.getInstance().updateDownloadTask(task);
							log.debug("runApplyTask.run", "自助指标查询 " + task.getId() + " 审核状态已更新：" + task.getApplyStatus() + " / " + task.getApplyPermitted());
						}
						if (conditionUpdate) { // 如果可以更新状态
							if (taskConditions != null && taskConditions.size() > 0) {
								for (FlatQueryTaskCondition condition : taskConditions) {
									if (condition.getTaskId().equals(task.getId())) {
										condition.setBuildPermitted(Global.YES);
										log.debug("runApplyTask.run", "自助指标查询 " + task.getId() + " 已允许生成");
										break;
									} // IF OVER
								} // FOR OVER
							} // IF OVER
						} else if (conditionRemove) { // 如果可以移除
							DownloadTaskServiceImpl.getInstance().updateDownloadTask(task);
							if (taskConditions != null && taskConditions.size() > 0) {
								for (FlatQueryTaskCondition condition : taskConditions) {
									if (condition.getTaskId().equals(task.getId())) {
										taskConditions.remove(condition);
										log.debug("runApplyTask.run", "自助指标查询 " + task.getId() + " 已从待生成队列移除");
										break;
									} // IF OVER
								} // FOR OVER
							} // IF OVER
						} // IF ELSE OVER
					}
				} // FOR OVER
			} // RUN OVER
		}, interval * 1000, interval * 1000);// 配置中的值为毫秒
	}


}
