package com.gic.cloud.data.hook.web;

import com.gic.cloud.common.api.base.Page;
import com.gic.cloud.data.hook.api.dto.DownloadRecord;
import com.gic.cloud.data.hook.api.dto.DownloadTask;
import com.gic.cloud.data.hook.api.dto.RiskModeRecord;
import com.gic.cloud.data.hook.api.entity.GeneralResult;
import com.gic.cloud.data.hook.api.service.IDownloadTaskService;
import com.gic.web.common.utils.SessionContextUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/** 下载任务相关
 * @author Sam
 */
@Controller
public class DownloadTaskController {

	/** 下载任务查询服务*/
	@SuppressWarnings("SpringJavaInjectionPointsAutowiringInspection")
	@Autowired
	private IDownloadTaskService downloadTaskService;

	/** 获取下载任务分页
	 * @param userId
	 * @param fuzzy
	 * @param pageNum
	 * @param pageSize
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/download-task-page")
	public Page<DownloadTask> downloadTaskPage(String userId, String fuzzy, Integer pageNum, Integer pageSize, HttpServletRequest request, HttpServletResponse response) {
		return this.downloadTaskService.getDownloadTaskPage(userId, fuzzy, pageNum, pageSize);
	}

	/** 删除下载任务
	 * @param taskId
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/delete-download-task")
	public GeneralResult deleteDownloadTask(String taskId, HttpServletRequest request, HttpServletResponse response) {
		GeneralResult result = new GeneralResult();
		boolean proResult = this.downloadTaskService.deleteDownloadTask(taskId);
		result.setResult(proResult);
		return result;
	}

	/** 创建下载日志
	 * @param taskId
	 * @param userId
	 * @param userName
	 * @param enterpriseId
	 * @return
	 */
	@RequestMapping("/create-download-record")
	public GeneralResult createDownloadRecord(String taskId, String userId, String userName, String enterpriseId) {
		GeneralResult result = new GeneralResult();
		Integer proResult = this.downloadTaskService.createDownloadRecord(taskId, userId, userName, enterpriseId);
		result.setResult(proResult == 1);
		return result;
	}

	private static SimpleDateFormat datetimeFormatter = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");

	/** 创建下载记录
	 * @param userId
	 * @param enterpriseId
	 * @param fuzzy
	 * @param pageNum
	 * @param pageSize
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/download-record-page")
	public Page<DownloadRecord> downloadTaskPage(String userId, String enterpriseId, String startTimeText, String endTimeText, Integer queryDataType, String fuzzyRange, String fuzzy, Integer pageNum, Integer pageSize, HttpServletRequest request, HttpServletResponse response) throws ParseException {
		String finStartTimeText = StringUtils.isNoneBlank(startTimeText) ? startTimeText + " 00:00:00" : "";
		String finEndTimeText = StringUtils.isNoneBlank(endTimeText) ? endTimeText + " 23:59:59" : "";
		Date startTime = finStartTimeText.equals("") ? null : datetimeFormatter.parse(finStartTimeText);
		Date endTime = finEndTimeText.equals("") ? null : datetimeFormatter.parse(finEndTimeText);
		if(SessionContextUtils.getLoginUser().getSuperAdmin() == 1){
			userId = null;
		}
		return this.downloadTaskService.getDownloadRecordPage(userId, enterpriseId, startTime, endTime, queryDataType, fuzzyRange, fuzzy, pageNum, pageSize);
	}

	/** 创建风险模式记录
	 * @param record
	 * @return
	 */
	@RequestMapping("/create-risk-mode-record")
	public GeneralResult createRiskModeRecord(RiskModeRecord record) {
		GeneralResult result = new GeneralResult();
		this.downloadTaskService.createRiskModeRecord(record);
		result.setResult(true);
		return result;
	}

	@RequestMapping("/risk-mode-record-page")
	public Page<RiskModeRecord> riskModeRecordPage(String enterpriseId, String startTimeText, String endTimeText, Integer riskDuration, String fuzzyRange, String fuzzy, Integer pageNum, Integer pageSize, HttpServletRequest request, HttpServletResponse response) throws ParseException {
		String finStartTimeText = StringUtils.isNoneBlank(startTimeText) ? startTimeText + " 00:00:00" : "";
		String finEndTimeText = StringUtils.isNoneBlank(endTimeText) ? endTimeText + " 23:59:59" : "";
		Date startTime = finStartTimeText.equals("") ? null : datetimeFormatter.parse(finStartTimeText);
		Date endTime = finEndTimeText.equals("") ? null : datetimeFormatter.parse(finEndTimeText);
		return this.downloadTaskService.getRiskModeRecordPage(enterpriseId, startTime, endTime, riskDuration, fuzzyRange, fuzzy, pageNum, pageSize);
	}



}
